//--------------------------------------------------------------------//
// biblioteka obsugi wywietlacza LCD ze sterownikiem Hitachi        //
// magistrala danych 4-bit, linie E, RS, zasilanie ukadu 3V          //
//--------------------------------------------------------------------//
                                                                      // pliki nagwkowe
#include "io430.h"                                                    // rejestry procesora 
#include "intrinsics.h"                                               // instrukcje procesora 
                                                                      //
                                                                      // interfejs mc <=> lcd  
                                                                      // definiuj linie we-wy  
#define          E_BIT               BIT1                             // linia E bit
#define          E_DIR               P1DIR                            // linia E kierunek 
#define          E_OUT               P1OUT                            // linia E wyjcie
                                                                      //
#define          RS_BIT              BIT0                             // linia RS bit
#define          RS_DIR              P1DIR                            // linia RS kierunek
#define          RS_OUT              P1OUT                            // linia RS wyjcie
                                                                      //
#define          D7_BIT              BIT0                             // linie magistrali 
#define          D7_DIR              P2DIR                            // danych
#define          D7_OUT              P2OUT                            // linie D7-D4 bit
#define          D6_BIT              BIT5                             // linie D7-D4 kierunek
#define          D6_DIR              P2DIR                            // linie D7-D4 wyjcie
#define          D6_OUT              P2OUT                            //
#define          D5_BIT              BIT6                             //
#define          D5_DIR              P2DIR                            //
#define          D5_OUT              P2OUT                            //
#define          D4_BIT              BIT1                             //
#define          D4_DIR              P2DIR                            //
#define          D4_OUT              P2OUT                            //   
                                                                      //
#define          CZESTOTLIWOSC_MCLK       1000000                     // czstotliwo zegara MCLK 
#define          TAKTY_ZEGARA_1_MS       (CZESTOTLIWOSC_MCLK/1000)    // takty 1 milisekunda   
#define          TAKTY_ZEGARA_100_US     (CZESTOTLIWOSC_MCLK/10000)   // takty 100 mikrosekund
                                                                      //
                                                                      //
void LcdPiny()                                                        // konfiguracja  
{                                                                     // linii we-wy    
   E_OUT  &= ~ E_BIT;                                                 // E, stan niski
   E_DIR  |=   E_BIT;                                                 // kierunek wyjcie  
   RS_OUT &= ~ RS_BIT;                                                // RS, stan niski
   RS_DIR |=   RS_BIT;                                                // kierunek wyjcie
                                                                      //
                                                                      // magistrala danych 
   D7_OUT &= ~ D7_BIT;                                                // D7, stan niski
   D7_DIR |=   D7_BIT;                                                // kierunek wyjcie
   D6_OUT &= ~ D6_BIT;                                                // D6, stan niski
   D6_DIR |=   D6_BIT;                                                // kierunek wyjcie
   D5_OUT &= ~ D5_BIT;                                                // D5, stan niski
   D5_DIR |=   D5_BIT;                                                // kierunek wyjcie        
   D4_OUT &= ~ D4_BIT;                                                // D4, stan niski
   D4_DIR |=   D4_BIT;                                                // kierunek wyjcie
}                                                                     //
                                                                      //
void LcdUstawLinie_D7_D4(unsigned char bajt)                          // ustaw linie danych D7-D4
{                                                                     //
   D7_OUT &= ~ D7_BIT;                                                // kada linia danych
   if(bajt & BIT7)   D7_OUT |=  D7_BIT;                               // jest zerowana, 
   D6_OUT &= ~ D6_BIT;                                                // nastpnie jest 
   if(bajt & BIT6)   D6_OUT |=  D6_BIT;                               // ustawiany stan linii: 
   D5_OUT &= ~ D5_BIT;                                                // niski, albo wysoki
   if(bajt & BIT5)   D5_OUT |=  D5_BIT;                               // stan linii okrela
   D4_OUT &= ~ D4_BIT;                                                // zmienna bajt
   if(bajt & BIT4)   D4_OUT |=  D4_BIT;                               // bity B7-B4 zmiennej 
                                                                      // definiuj ustawienie
}                                                                     // linii danych D7-D4
                                                                      //
void LcdWyslijInstrukcje (unsigned char  instrukcja)                  // wylij instrukcj
{                                                                     // 
    RS_OUT  &= ~ RS_BIT;                                              // linia RS,      
                                                                      // stan niski 
    LcdUstawLinie_D7_D4(instrukcja);                                  // ustaw dane, bity 7-4
    E_OUT |=  E_BIT;                                                  // strobuj E
    E_OUT &=~ E_BIT;                                                  // czekaj czas wykonania
  __delay_cycles(TAKTY_ZEGARA_100_US  * 1);                           // instrucji 100us
                                                                      // przesu o 4 bity w lewo
    LcdUstawLinie_D7_D4(instrukcja << 4);                             // ustaw dane, bity 7-4   
    E_OUT |=  E_BIT;                                                  // strobuj E
    E_OUT &=~ E_BIT;                                                  // czekaj czas wykonania
  __delay_cycles(TAKTY_ZEGARA_100_US  * 1);                           // instrucji 100us
}                                                                     // 
                                                                      //  
void LcdWyslijDane(unsigned char dane)                                // wylij dane
{                                                                     //
    RS_OUT  |= RS_BIT;                                                // linia RS, 
                                                                      // stan wysoki
    LcdUstawLinie_D7_D4(dane);                                        // ustaw dane, bity 7-4
    E_OUT |=  E_BIT;                                                  // strobuj E
    E_OUT &=~ E_BIT;                                                  // czekaj 100us   
  __delay_cycles(TAKTY_ZEGARA_100_US  * 1);                           //  
                                                                      //  
    LcdUstawLinie_D7_D4(dane << 4);                                   // przesu o 4 bity w lewo
    E_OUT |=  E_BIT;                                                  // ustaw dane, bity 7-4  
    E_OUT &=~ E_BIT;                                                  // strobuj E
  __delay_cycles(TAKTY_ZEGARA_100_US  * 1);                           // czekaj 100us 
}                                                                     //  
                                                                      //    
                                                                      //
void  LcdSekwencjaStartowa4b3V()                                      // sekwencja startowa
{                                                                     // interfejs 4-bit,Vcc 3V 
    E_OUT  &= ~ E_BIT;						      //	
    RS_OUT &= ~ RS_BIT;						      // 																		
  								      // czekaj 40ms 							
  __delay_cycles(TAKTY_ZEGARA_1_MS * 40);                             // zerowanie sterownika LCD
								      //							
    LcdUstawLinie_D7_D4(0x30);                                        // 1szy raz ustaw 0x30
    E_OUT |=  E_BIT;                                                  // strobuj 
    E_OUT &=~ E_BIT;                                                  // sygna E                                                                        
  __delay_cycles(TAKTY_ZEGARA_1_MS * 5);                              // czekaj 5ms
                                                                      //
    LcdUstawLinie_D7_D4(0x30);                                        // 2gi raz ustaw 0x30     
    E_OUT |=  E_BIT;                                                  // strobuj 
    E_OUT &=~ E_BIT;                                                  // sygna E
  __delay_cycles(TAKTY_ZEGARA_1_MS * 5);                              // czekaj 5ms
                                                                      //
    LcdUstawLinie_D7_D4(0x30);                                        // 3ci raz ustaw 0x30
    E_OUT |=  E_BIT;                                                  // strobuj  
    E_OUT &=~ E_BIT;                                                  // sygna E
  __delay_cycles(TAKTY_ZEGARA_1_MS * 5);                              // czekaj 5ms
                                                                      //  
    LcdUstawLinie_D7_D4(0x20);                                        // ustaw 0x20
    E_OUT |=  E_BIT;                                                  // strubuj 
    E_OUT &=~ E_BIT;                                                  // sygna E
  __delay_cycles(TAKTY_ZEGARA_1_MS * 5);                              // czekaj 5ms    
                                                                      // przejcie w tryb
}                                                                     // pracy 4-bit
                                                                      //    
void  LcdInicjalizacja()                                              // inicjalizuj LCD   
{                                                                     // 0x28, interfejs 4-bit,   
    LcdWyslijInstrukcje(0x28);                                        // 2 linie, matryca 5x7
    LcdWyslijInstrukcje(0x0C);                                        // 0x0C, wcz lcd, 
                                                                      // wycz kursor 
    LcdWyslijInstrukcje(0x01);                                        // 0x01, czy pami ekranu  
  __delay_cycles(TAKTY_ZEGARA_1_MS * 2);                              // czekaj 2ms
    LcdWyslijInstrukcje(0x06);                                        // 0x06, zapis do pamici
                                                                      // DD-RAM, CG_RAM, zwiksza
                                                                      // o jeden adres pamici 
}                                                                     //
                                                                      //  
void LcdWyczyscEkran()                                                // czy pami ekranu
{                                                                     // DD_RAM
    LcdWyslijInstrukcje(0x01);                                        // wylij instukcje   
  __delay_cycles(TAKTY_ZEGARA_1_MS * 2);                              // czekaj 2ms   
}                                                                     //
                                                                      //  
void  LcdUstawKursorWK(unsigned char wiersz, unsigned char kolumna)   // ustaw adres pamici
{                                                                     // ekranu DD_RAM   
    LcdWyslijInstrukcje(0x80 + (wiersz* 0x40 + kolumna));             // obliczenie adresu  
}                                                                     // wysanie instrukcji
                                                                      //
void LcdPiszZnak(unsigned char   kod)                                 // pisz znak do pamici 
{                                                                     // ekranu DD_RAM, 
                                                                      // znak identyfikowany  
                                                                      // przez kod, jest przesyany 
                                                                      // z pamici CG_ROM/CG_RAM
    LcdWyslijDane(kod);                                               // do pamici ekranu DD_RAM
                                                                      // po wpisaniu znaku 
                                                                      // adres pamici jest
}                                                                     // automatycznie zwikszany
                                                                      //        
void LcdPiszTekst(unsigned char * tekst)                              // pisz tekst do pamici
{                                                                     // ekranu DD_RAM
     while(*tekst !=0)                                                // dopki nie ma koca    
         LcdPiszZnak(*tekst ++);                                      // tekstu (warto 0)   
                                                                      // pisz kolejne znaki 
}                                                                     //
                                                                      //
void LcdUstawAdresDefZnaku(unsigned char kod, unsigned char komorka)  // ustaw adres pamici
{                                                                     // CG_RAM (kod,komorka)  
     LcdWyslijInstrukcje(0x40 + (kod<<3) + komorka);                  // obliczenie adresu
                                                                      // wysanie instrukcji
}                                                                     //         
                                                                      //                                        
void  LcdDefiniujZnak(unsigned char bajt)                             // zapis danych do pamici
{                                                                     // CG_RAM (bajt danych) 
                                                                      // wprowadzone dane definij   
     LcdWyslijDane(bajt);                                             // matryc znaku
                                                                      // po wpisaniu bajta danych 
                                                                      // adres pamici jest
}                                                                     // zwikszany automatycznie
                                                                      //            