//   "MSP430 z pamici FRAM"                                         // konfiguracja "MSP-EXP430FR5739": 
//   Zadanie numer 7.1. Obsuga analogowego wejcia. Termistor.       // ( ustawienia fabryczne )
//                                                                    // w programie wykorzystano
//                              ACLK   nieaktywny                     // zainstalowany w module 
//                              MCLK = SMCLK = DCOCLK ~1MHz           // termistor NTC, 100 kOm
//                                                                    // do moduu podczamy wywietlacz
//                           MSP430FR5739                             // 
//                           -----------------                        // dziaanie programu:
//                         |                 |                        // raz na sekund wykonywany jest pomiar 
//                         |            P1.0 | -> LCD  E              // napicia na wyjciu termistora NTC
//                         |            P1.1 | -> LCD RS              // wynik pomiaru konwertowany jest na 
//                         |            P2.0 | -> LCD D7              // warto temperatury w st. Celsjusza
//                         |            P2.5 | -> LCD D6              // na ekranie LCD wywietlane s:
//                  A4 ->  | P1.4       P2.6 | -> LCD D5              // wynik pomiaru binarnie i w Voltach   
//          NTC on/off <-  | P2.7       P2.1 | -> LCD D4     .K.'13  // oraz temperatura w st. Celsjusza     
//------------------------------------------------------------------- // 
                                                                      // pliki nagwkowe
#include "io430.h"                                                    // rejestry procesora 
#include "intrinsics.h"                                               // instrukcje procesora
#include "LCD_2x16.h"                                                 // procedury LCD 
#include "tlv.h"                                                      // struktura TLV                 
#include <stdio.h>                                                    // funkcje wejcia/wyjcia  
                                                                      //
                                                                      // zmienne globalne                                              
unsigned int  wPomiaruB;                                              // wynik pomiaru (binarnie)   
float         wPomiaruV;                                              // wynik pomiaru (Volty) 
float         wPomiaruC;                                              // wynik pomiaru (stopnie Celsjusza)          
                                                                      //
void main( void )                                                     // program gwny     
{                                                                     //
  WDTCTL = WDTPW + WDTHOLD;                                           // zatrzymaj ukad Watchdog 
                                                                      // 
  unsigned char tekst[16 + 1];                                        // zmienna lokalna 
                                                                      // bufor, tekst ekranu LCD
                                                                      // 
                                                                      // konfiguracja linii we/wy 
  P1OUT = 0x00; P2OUT = 0x00; P3OUT = 0x00; P4OUT = 0x00;             // stan niski na liniach 
  PJOUT = 0x00;                                                       // linie w kierunku wyjcia       
  P1DIR = 0xff; P2DIR = 0xff; P3DIR = 0xff; P4DIR = 0xff;             // wszystkie linie to:    
  PJDIR = 0xff;                                                       // wyjcia w stanie niskim 
                                                                      //
                                                                      // konfiguracja LCD 
  LcdPiny();                                                          // linie we/wy  
  LcdSekwencjaStartowa4b3V();                                         // sekwencja startowa 
  LcdInicjalizacja();                                                 // inicjalizacja LCD
                                                                      //
                                                                      // konfiguracja generatora REF
  while(REFCTL0 & REFGENBUSY);                                        // czekaj dopki gen. jest zajty                                           
  REFCTL0  = REFVSEL_0 + REFTCOFF;                                    // ustaw napicie VREF =  1.5V 
                                                                      // wycz czujnik temperatury  
                                                                      //
  P1SELC |= BIT4;                                                     // konfiguracja  
  P1SEL1 |= BIT4;                                                     // wejcia analogowego A4  
  P1SEL0 |= BIT4;                                                     // ustaw trzeci funkcyjny tryb pracy  
                                                                      //
                                                                      // konfiguracja przetwornika ADC
  ADC10CTL0 &=~ ADC10ENC;                                             // wyzeruj bit zezwolenia na konwersj                          
                                                                      // prbkowanie 32 takty ADC10CLK
  ADC10CTL0  = ADC10SHT_3;                                            // ustaw tryb pojedynczego pomiaru
  ADC10CTL1  = ADC10SHP + ADC10SHS_0 + ADC10SSEL_0 + ADC10CONSEQ_0;   // taktowanie pomiaru z wew. osc. 5MHz
  ADC10CTL2  = ADC10RES;                                              // rozdzielczo pomiaru 10 bitw
  ADC10MCTL0 = ADC10SREF_1 + ADC10INCH_4;                             // potencjay referencyjne: 
                                                                      // V(R+) = VREF,  V(R-) = AVSS
                                                                      // obsuga zew. kanau pomiarowego nr. 4
                                                                      //
  ADC10IFG  &=~ ADC10IFG0;                                            // wyzeruj flag przerwania  
  ADC10IE   |=  ADC10IE0;                                             // wcz obsug przerwa 
                                                                      // od zakoczenia pomiaru napicia  
                                                                      //
 __bis_SR_register(GIE);                                              // odblokuj obsug  
                                                                      // przerwa maskowalnych
                                                                      //      
 while(1)                                                             // ptla gwna programu 
 {                                                                    //                                                          
     while(REFCTL0 & REFGENBUSY);                                     // czekaj dopki gen. REF jest zajty                                           
     REFCTL0    |= REFON;                                             // wcz generator  
     P2OUT      |= BIT7;                                              // wcz zasilanie termistora NTC
     ADC10CTL0  |= ADC10ON;                                           // wcz przetwornik ADC10
                                                                      // 
     __delay_cycles(100);                                             // czekaj 100us   
                                                                      // 
     ADC10CTL0  |= ADC10ENC;                                          // ustaw bit zezwolenia na konwersj
     ADC10CTL0  |= ADC10SC;                                           // rozpocznij pomiar napicia
                                                                      // ( programowe wyzwalanie pomiaru )
                                                                      //
     __bis_SR_register(CPUOFF );                                      // przejd w tryb upienia LPM0                                          
                                                                      // koniec pomiaru, wyjcie z upienia
                                                                      //  
     P2OUT      &=~ BIT7;                                             // wycz zasilanie termistora NTC                                                                                          
     ADC10CTL0  &=~ ADC10ENC;                                         // wyzeruj bit zezwolenia na konwersj 
     ADC10CTL0  &=~ ADC10ON;                                          // wycz przetwornik ADC10          
     while(REFCTL0 & REFGENBUSY);                                     // czekaj dopki gen. REF jest zajty                                            
     REFCTL0    &=~ REFON;                                            // wycz generator napicia referencyjnego
                                                                      //
                                                                      // kalibracja binarnej wartoci pomiaru
     wPomiaruB = (unsigned int)(wPomiaruB * (float)(sTlvRef.CAL_ADC_15VREF_FACTOR / 32768)); //  ( wzr 7.9 ) 
     wPomiaruB = (unsigned int)(wPomiaruB * (float)(sTlvAdc10.CAL_ADC_GAIN_FACTOR / 32768)) + sTlvAdc10.CAL_ADC_OFFSET;
                                                                      //
     wPomiaruV = (float)(wPomiaruB * 1.5) / 1023;                     // obliczenie wartoci w Voltach  ( wzr 7.4 )
     wPomiaruC = (float)(0.4542 - wPomiaruV)/0.0088;                  // obliczenie wartoci w st. Celsjusza ( wzr 7.16 )
                                                                      //
                                                                      // prezentacja wyniku
                                                                      //  
     LcdWyczyscEkran();                                               // wyczy ekran LCD
     LcdUstawKursorWK(0,0);                                           // ustaw kursor na pocztku 1 linii
     snprintf((char*)tekst,17,"B: %.4d A: %.2fV",wPomiaruB,wPomiaruV);// wypisz wynik pomiaru prezentowany  
     LcdPiszTekst(tekst);                                             // binarnej oraz w Voltach  
                                                                      // nastpnie  
     LcdUstawKursorWK(1,0);                                           // ustaw kursor na pocztku 2 linii
     snprintf((char*)tekst,17,"Temp: %.1f%cC",wPomiaruC,223);         // wypisz wynik pomiaru prezentowany 
     LcdPiszTekst(tekst);                                             // w stopniach Celsjusza                                         
                                                                      //
     __delay_cycles(1000000);                                         // czekaj 1 sekund          
  };                                                                  // 
}                                                                     //    
                                                                      //
#pragma vector=ADC10_VECTOR                                           // procedura obsugi przerwania 
__interrupt void ADC10_ISR(void)                                      // przetwornika A/C, modu ADC10
{                                                                     //  
  switch(__even_in_range(ADC10IV,12))                                 // odczytaj rejestr ADC10IV                                                                 
  {                                                                   //  
    case  0: break;                                                   // brak przerwa
    case  2: break;                                                   // nadpisanie sekwencji pomiarowej
    case  4: break;                                                   // nadpisanie wyniku pomiaru
    case  6: break;                                                   // komparator analogowy, prg w gr
    case  8: break;                                                   // komparator analogowy, prg w d
    case 10: break;                                                   // komparator analogowy, prg pomidzy
                                                                      // 
    case 12:                                                          // zakoczenia pomiaru                                                          
             wPomiaruB = ADC10MEM0;                                   // odczytaj wynik pomiaru     
             __bic_SR_register_on_exit(CPUOFF);                       // wyjd z trybu upienia 
             break;                                                   //    
    default: break;                                                   // brak obsugi                                                  
  }                                                                   //                                                                          
}                                                                     //


